@@ -5,16 +5,34 @@ All rights reserved.
 import logging
 import os
 import posixpath
-import textwrap
 
 from mkdocs import utils
 from mkdocs.config import config_options
 from mkdocs.plugins import BasePlugin
+from mkdocs.structure.files import File
 
 log = logging.getLogger('mkdocs.plugin.redirects')
 log.addFilter(utils.warning_filter)
 
 
+HTML_TEMPLATE = """
+<!doctype html>
+<html lang="en">
+<head>
+    <meta charset="utf-8">
+    <title>Redirecting...</title>
+    <link rel="canonical" href="{url}">
+    <meta name="robots" content="noindex">
+    <script>var anchor=window.location.hash.substr(1);location.href="{url}"+(anchor?"#"+anchor:"")</script>
+    <meta http-equiv="refresh" content="0; url={url}">
+</head>
+<body>
+Redirecting...
+</body>
+</html>
+"""
+
+
 def write_html(site_dir, old_path, new_path):
     """Write an HTML file in the site_dir with a meta redirect to the new page"""
     # Determine all relevant paths
@@ -28,26 +46,9 @@ def write_html(site_dir, old_path, new_path):
         os.makedirs(old_dir_abs)
 
     # Write the HTML redirect file in place of the old file
+    log.debug("Creating redirect: '%s' -> '%s'", old_path, new_path)
+    content = HTML_TEMPLATE.format(url=new_path)
     with open(old_path_abs, 'w', encoding='utf-8') as f:
-        log.debug("Creating redirect: '%s' -> '%s'", old_path, new_path)
-        content = textwrap.dedent(
-            """
-            <!doctype html>
-            <html lang="en">
-            <head>
-                <meta charset="utf-8">
-                <title>Redirecting...</title>
-                <link rel="canonical" href="{url}">
-                <meta name="robots" content="noindex">
-                <script>var anchor=window.location.hash.substr(1);location.href="{url}"+(anchor?"#"+anchor:"")</script>
-                <meta http-equiv="refresh" content="0; url={url}">
-            </head>
-            <body>
-            Redirecting...
-            </body>
-            </html>
-            """
-        ).format(url=new_path)
         f.write(content)
 
 
@@ -55,14 +56,8 @@ def get_relative_html_path(old_page, new_page, use_directory_urls):
     """Return the relative path from the old html path to the new html path"""
     old_path = get_html_path(old_page, use_directory_urls)
     new_path, new_hash_fragment = _split_hash_fragment(new_page)
-    new_path = get_html_path(new_path, use_directory_urls)
-
-    if use_directory_urls:
-        # remove /index.html from end of path
-        new_path = posixpath.dirname(new_path) or './'
 
     relative_path = posixpath.relpath(new_path, start=posixpath.dirname(old_path))
-
     if use_directory_urls:
         relative_path = relative_path + '/'
 
@@ -71,26 +66,8 @@ def get_relative_html_path(old_page, new_page, use_directory_urls):
 
 def get_html_path(path, use_directory_urls):
     """Return the HTML file path for a given markdown file"""
-    parent, filename = posixpath.split(path)
-    name_orig = posixpath.splitext(filename)[0]
-
-    # Both `index.md` and `README.md` files are normalized to `index.html` during build
-    name = 'index' if name_orig in ('index', 'README') else name_orig
-
-    # Directory URLs require some different logic. This mirrors mkdocs' internal logic.
-    if use_directory_urls:
-
-        # If it's name is `index`, then that means it's the "homepage" of a directory, so should get placed in that dir
-        if name == 'index':
-            return posixpath.join(parent, 'index.html')
-
-        # Otherwise, it's a file within that folder, so it should go in its own directory to resolve properly
-        else:
-            return posixpath.join(parent, name, 'index.html')
-
-    # Just use the original name if Directory URLs aren't used
-    else:
-        return posixpath.join(parent, (name + '.html'))
+    f = File(path, '', '', use_directory_urls)
+    return f.dest_path.replace(os.sep, '/')
 
 
 class RedirectPlugin(BasePlugin):
@@ -103,13 +80,6 @@ class RedirectPlugin(BasePlugin):
     def on_files(self, files, config, **kwargs):
         self.redirects = self.config.get('redirect_maps', {})
 
-        # SHIM! Produce a warning if the old root-level 'redirects' config is present
-        if config.get('redirects'):
-            log.warning(
-                "The root-level 'redirects:' setting is not valid and has been changed in version 1.0! "
-                "The plugin-level 'redirect-map' must be used instead. See https://git.io/fjdBN"
-            )
-
         # Validate user-provided redirect "old files"
         for page_old in self.redirects.keys():
             if not utils.is_markdown_file(page_old):
@@ -129,14 +99,15 @@ class RedirectPlugin(BasePlugin):
         # Walk through the redirect map and write their HTML files
         for page_old, page_new in self.redirects.items():
             # Need to remove hash fragment from new page to verify existence
-            page_new_without_hash, _ = _split_hash_fragment(str(page_new))
+            page_new_without_hash, hash = _split_hash_fragment(str(page_new))
 
             # External redirect targets are easy, just use it as the target path
             if page_new.lower().startswith(('http://', 'https://')):
                 dest_path = page_new
 
             elif page_new_without_hash in self.doc_pages:
-                dest_path = get_relative_html_path(page_old, page_new, use_directory_urls)
+                file = self.doc_pages[page_new_without_hash]
+                dest_path = get_relative_html_path(page_old, file.url + hash, use_directory_urls)
 
             # If the redirect target isn't external or a valid internal page, throw an error
             # Note: we use 'warn' here specifically; mkdocs treats warnings specially when in strict mode
